
----  M E A D E R  T A C T I C S  ----
----  O P T I O N S    P A N E L  ----
--[[
----  Current Features  ----
* Change button text size (radio)
* Change growth direction for boss and message buttons (raido)

----  Planned Features  ----
* Tabs: Visuals, messages, quickbar, import/export

* enter and leave test mode (onOffButton)
* export and import
* change button sizes and/or scale of frames (slider)
* lock posisions of frames (onOffButton) ??
* quick bar key bindings (special)
* delete tactic-set (button)
* reset options to default values (button)
--]]

---- INITIALIZE ----
-- Global function lib
meaderOptions = {}

--Function pre-define
local init
local createOptionTabs
local switchTabs
local optionsPanelFrame
local fillOptionsTab
local saveOption
local createRadioButton
local uncheckRadioButtons

-- Pre-define the main tables
local tabDB = {}
local optionDB = {}
local valueDB = {}
local tooltipDB = {}
local setting

--The main object for frames. To be defined in createOptionsPanel
local optionsPanel = CreateFrame("Frame", "MeaderTacticsOptionPanel", UIParent) --, "MeaderTacticsPanelTemplate")
optionsPanel.created = false
optionsPanel.tabButton = {}

--Define global reference to the panel
--meaderOptionsPanel

-- Define throwaway tables
local tempTable = { [1] = {}, [2] = {} }  --Remember to allways do tempTable = {} before, and preferable after, using it.


-- TEMP SETTINGS

--setting.buttonTemplate = "MeaderTacticsButtonTemplate"
local currentTab = "Visuals"

--[[ 
-----------------------------------------
---- TEMPLATE FOR ADDING NEW OPTIONS ----

optionDB[#optionDB+1] = {
			ID = 0
			name = "",
			tabID = 0,
			order = 1,
			header = ""
			text = "",
			optionType = "",
			tooltipID = 0,
			valueID = 0,
			defaultValueID = "",
			maxValue = 0,
			minValue = 0,
			hidden = false
			}
valueDB[#valueDB+1] = {ID = 0, parentID=#optionDB, name="", text="", value="", order=1, tooltipID=0}
tooltipDB[#tooltipDB] = {ID = 0, parentID = 0, parentType = "", text = "", order = 1}

-----------------------------------------
--]]

-------------------
---- FUNCTIONS ----
-------------------



init = function()
	setting = meaderTacticsSetting
	if not optionsPanel.created then
		tabDB = meaderTacticsOptions["tabDB"]
		optionDB = meaderTacticsOptions["optionDB"]
		valueDB = meaderTacticsOptions["valueDB"]
		tooltipDB = meaderTacticsOptions["tooltipDB"]
		optionsPanelFrame()
	else
		optionsPanel:Show()
	end
end--init
meaderOptions.init = init

optionsPanelFrame = function()
	optionsPanel:SetSize(setting.optionsPanelWidth, setting.optionsPanelHeight)
	optionsPanel:SetPoint(setting.optionsPanelPointA, UIParent,setting.optionsPanelPointA, setting.optionsPanelPointX, setting.optionsPanelPointY)
	optionsPanel:SetMovable(true)
   	optionsPanel:EnableMouse(true)
   	optionsPanel:RegisterForDrag("LeftButton")
   	optionsPanel:SetFrameStrata("FULLSCREEN_DIALOG")
   	optionsPanel:SetToplevel(true)
   	optionsPanel:SetScript("OnDragStart", optionsPanel.StartMoving)
   	optionsPanel:SetScript("OnDragStop", optionsPanel.StopMovingOrSizing)

	local texture = optionsPanel:CreateTexture("BACKGROUND")
	texture:SetPoint("CENTER",0,0)
	texture:SetSize(setting.optionsPanelWidth-15, setting.optionsPanelHeight-15)
	texture:SetTexture(0,0,0,1)
	local backdrop = {
 			bgFile = "Interface/DialogFrame/UI-DialogBox-Background", --setting.optionsPanelBackdropBgFile,  
  			edgeFile = "Interface/Tooltips/UI-Tooltip-Border", --setting.optionsPanelEdgeFile,
  			tile = true,
  			tileSize = 32,
  			edgeSize = 15,
  			insets = {
    			left = 4,
    			right = 4,
    			top = 3,
    			bottom = 3
  			}
	}--]]
	optionsPanel:SetBackdrop(backdrop)
	
	-- Header
		local header = optionsPanel:CreateFontString()
		header:SetFont(STANDARD_TEXT_FONT,15, "OUTLINE")
		header:SetJustifyH("CENTER")
		header:SetJustifyV("CENTER")
		header:SetPoint("TOP", optionsPanel,"TOP", 0, 50)
		header:SetSize(setting.optionsPanelWidth, 25)
		header:SetTextColor(1, 0.3, 0, 1)  --MeaderRed
		header:SetText("Meader Tactics - Options")
	
	--Quit Button
   	optionsPanel.cancelButton = CreateFrame("Button", "meaderTacticsOptionPanelCancelButton", optionsPanel, setting.smallButtonTemplate)
   	optionsPanel.cancelButton:SetPoint("BOTTOMRIGHT", optionsPanel, "BOTTOMRIGHT", 0, -25)
   	optionsPanel.cancelButton:SetSize(100,25)
   	optionsPanel.cancelButton:SetText("Save and Quit")
   	optionsPanel.cancelButton:SetScript("OnClick", function(self)
   		meaderFrame.alertBox("Quiting the options panel, without reloadig your UI, may cause some weird things to happen. Most changes will probably not be seen before a reload.\n (If you have not opened Meader Tactics since last reload there is no need to reload now).\n\n\n Would you like to reload your UI now?", true, function() ReloadUI(); end, function()optionsPanel:Hide(); end)
   		optionsPanel:Hide()
	end)	
	--Reload UI Button
   	optionsPanel.saveButton = CreateFrame("Button", "meaderTacticsOptionPanelReloadButton", optionsPanel, setting.smallButtonTemplate)
   	optionsPanel.saveButton:SetPoint("BOTTOMRIGHT", optionsPanel, "BOTTOMRIGHT", -105, -25)
   	optionsPanel.saveButton:SetSize(125,25)
   	optionsPanel.saveButton:SetText("Save and Reload UI")
   	optionsPanel.saveButton:SetScript("OnClick", function(self)
   		ReloadUI()
	end)	

	optionsPanel.created = true
	createOptionTabs()
	optionsPanel:Show()
	
	meaderOptionsPanel = optionsPanel
end--createOptionPanel


createOptionTabs = function ()
	local tabButtonWidth = setting.optionsPanelWidth/#tabDB
	for tabKey,v in pairs(tabDB) do
		local tabID = tabDB[tabKey]["ID"]
		local tabName = tabDB[tabKey]["name"]
		--create tab frame
		optionsPanel[tabName] = CreateFrame("Frame", "MeaderTacticsOptionPanelTab"..tabName, optionsPanel)
		optionsPanel[tabName]:SetAllPoints()
		
		--create button
		optionsPanel.tabButton[tabName] = CreateFrame("Button", "MeaderTacticsOptionPanelTab"..tabName, optionsPanel, "MeaderTacticsButtonTemplate")
		optionsPanel.tabButton[tabName]:SetText(tabDB[tabKey]["text"])
		optionsPanel.tabButton[tabName]:SetSize(tabButtonWidth, 25)
		optionsPanel.tabButton[tabName]:SetPoint("TOPLEFT", optionsPanel,"TOPLEFT", (tabDB[tabKey]["order"]*tabButtonWidth)-tabButtonWidth, 25)
		--set onclick script
		optionsPanel.tabButton[tabName]:SetScript("OnClick", function(self) 
			switchTabs(tabName)
		end)
		fillOptionsTab(tabID)
		optionsPanel[tabName]:Hide()
	end--for
	switchTabs(tabDB[1]["name"])
end--createTabButtons

switchTabs = function (tabName)
	for tabKey, v in pairs(tabDB) do
		local tabID = tabDB[tabKey]["ID"]
		if tabDB[tabKey]["name"] == tabName then
			optionsPanel.tabButton[tabDB[tabKey]["name"]]:LockHighlight()
			optionsPanel[tabName]:Show()
		else
			optionsPanel.tabButton[tabDB[tabKey]["name"]]:UnlockHighlight()
			optionsPanel[tabDB[tabKey]["name"]]:Hide()
		end
	end
end--switchTabs

fillOptionsTab = function (tabID)
	local tabKey = getKey("tabDB", tabID)
	local tabName = tabDB[tabKey]["name"]
	tempTable[1] = {}
	
--DEBUG:
--print("tab key:"..tabKey.." name: "..tabName)
--END DEBUG

	for optionKey, v in pairs(optionDB) do
		local optionID = optionDB[optionKey]["ID"]
		if optionDB[optionKey]["tabID"] == tabID then
			local optionName = optionDB[optionKey]["name"]
			local height = 0
			--create option frame
			optionsPanel[tabName][optionName] = CreateFrame("Frame", "MeaderTacticsOptionPanel"..tabName.."Option"..optionName, optionsPanel[tabName])
			optionsPanel[tabName][optionName]:SetWidth(setting.optionsPanelWidth)
		
			-- create option fontstrings for Header and Text
			local header = optionsPanel[tabName][optionName]:CreateFontString()
			header:SetFont(STANDARD_TEXT_FONT,15, "OUTLINE")
			header:SetJustifyH("LEFT")
			header:SetPoint("TOPLEFT", optionsPanel[tabName][optionName],"TOPLEFT", 0, 0)
			header:SetWidth(setting.optionsPanelWidth-30)
			header:SetTextColor(1, 0.3, 0, 1) --MeaderRed
			header:SetText(optionDB[optionKey]["header"])
			height = height + header:GetStringHeight() + 8
			
			local text = optionsPanel[tabName][optionName]:CreateFontString()
			if optionDB[optionKey]["optionType"] == "text" then
				text:SetFont(STANDARD_TEXT_FONT,12, "OUTLINE")
			else
				text:SetFont(STANDARD_TEXT_FONT,10, "OUTLINE")
			end
			text:SetJustifyH("LEFT")
			text:SetPoint("TOPLEFT", optionsPanel[tabName][optionName],"TOPLEFT", 0, -1*(header:GetStringHeight()+5))
			text:SetWidth(setting.optionsPanelWidth-30)
			text:SetTextColor(1, 1, 1, 1) --MeaderRed
			text:SetWordWrap(true)
			text:SetText(optionDB[optionKey]["text"])
			height = height + text:GetStringHeight() + 8
			
			-- Option Type Selection
			if optionDB[optionKey]["optionType"] == "button" then
			elseif optionDB[optionKey]["optionType"] == "slider" then
			elseif optionDB[optionKey]["optionType"] == "checkbox" then
			elseif optionDB[optionKey]["optionType"] == "radiobutton" then
				--retrieve optionValues
				tempTable[2] = {}
				for valueKey, v in pairs(valueDB) do
					local valueID = valueDB[valueKey]["ID"]
					if valueDB[valueKey]["parentID"] == optionID then
						local buttonName = valueDB[valueKey]["name"]
						createRadioButton(valueID, optionID, tabName)
						tempTable[2][valueDB[valueKey]["order"]] = valueKey 
					end
				end--for optionValue
				
				--Set points according to order
				local hpoint = 0
				for order, valueKey in ipairs(tempTable[2]) do
					local buttonName = valueDB[valueKey]["name"]
					--make sure the width does not go beyond the frame, if so move to next "line"
					local radioWidth = _G[optionsPanel[tabName][optionName][buttonName]:GetName() .. "Text"]:GetStringWidth() + 50
					if hpoint+radioWidth > setting.optionsPanelWidth-30 then hpoint = 0; height = height + 25 end
					--set point					
					optionsPanel[tabName][optionName][buttonName]:SetPoint("TOPLEFT", hpoint, -height)
					hpoint = hpoint + radioWidth
				end
				tempTable[2] = {}

				height = height + 25
			elseif optionDB[optionKey]["optionType"] == "onoffbutton" then
			elseif optionDB[optionKey]["optionType"] == "inputbox" then
			elseif optionDB[optionKey]["optionType"] == "text" then
			elseif optionDB[optionKey]["optionType"] == "special" then
			end--Option Type Selection
			
		optionsPanel[tabName][optionName]:SetHeight(height)
		tempTable[1][optionDB[optionKey]["order"]] = optionName
		end--if tabID
	end--for option
	--Set points according to order
	local nextPos = -25
	for k,v in ipairs(tempTable[1]) do
		local h = optionsPanel[tabName][v]:GetHeight()
		optionsPanel[tabName][v]:SetPoint("TOPLEFT", 15, nextPos)
		nextPos = nextPos - h - 5	
	end
	tempTable[1] = {}
end--fillOptionsTab

createRadioButton = function(valueID, optionID, tabName)
	local optionKey = getKey("optionDB", optionID)
	local valueKey = getKey("valueDB", valueID)
	local optionName = optionDB[optionKey]["name"]
	local buttonName = valueDB[valueKey]["name"]
	local parentID = valueDB[valueKey]["parentID"]
	local parentKey = getKey("optionDB", parentID )
	local checked = false

	if optionDB[parentKey]["valueID"]==valueID then checked = true end
	--Create a new button.
	local radioButton = CreateFrame("CheckButton", "MeaderTacticsOptionPanel"..tabName..optionName..buttonName, optionsPanel[tabName][optionName], "UIRadioButtonTemplate")  --"InterfaceOptionsCheckButtonTemplate") <-- for CHECKBOX only 
	radioButton:SetHeight(20)
	radioButton:SetWidth(20)
	radioButton:SetChecked(checked)
	_G[radioButton:GetName() .. "Text"]:SetText(valueDB[valueID]["text"])
	radioButton:SetScript("OnClick", function(self) 
		self:SetChecked(true)
		optionDB[optionKey]["valueID"] = valueID
		saveOption(optionKey, valueKey)
		uncheckRadioButtons(optionID, tabName)
	end)
	optionsPanel[tabName][optionName][buttonName] = radioButton
end--createRadioButton

uncheckRadioButtons = function(optionID, tabName)
	local optionKey = getKey("optionDB", optionID)
	for valueKey, v in pairs(valueDB) do
		local valueID = valueDB[valueKey]["ID"]
		if valueDB[valueKey]["parentID"] == optionID then
			optionsPanel[tabName][optionDB[optionKey]["name"]][valueDB[valueKey]["name"]]:SetChecked(false)

			if optionDB[optionKey]["valueID"] == valueID then
				optionsPanel[tabName][optionDB[optionKey]["name"]][valueDB[valueKey]["name"]]:SetChecked(true)
			end
		end--if parentID
	end--for
end--uncheckRadioButtons


saveOption = function(optionKey, valueKey)
		local name = optionDB[optionKey]["name"]
		local value = valueDB[valueKey]["value"]
		meaderTacticsSetting[name] = value
end--saveOption

getKey = function(tableName, ID)
	if tableName == "tabDB" then
		for key, v in pairs(tabDB) do
			if tabDB[key]["ID"] == ID then return key end
		end
	end
	if tableName == "optionDB" then
		for key, v in pairs(optionDB) do
			if optionDB[key]["ID"] == ID then return key end
		end
	end
	if tableName == "valueDB" then
		for key, v in pairs(valueDB) do
			if valueDB[key]["ID"] == ID then return key end
		end
	end
	if tableName == "tooltipDB" then
		for key, v in pairs(tooltipDB) do
			if tooltipDB[key]["ID"] == ID then return key end
		end
	end
end


--[[
--Remnants of the test preiod
SLASH_MEADEROPTIONS1 = "/meaderopt"
function SlashCmdList.MEADEROPTIONS(...)
	arg1 = ...
	if not optionsPanel.created then
		init()
		--optionsPanelFrame()
	else
		optionsPanel:Show()
	end
end
--]]
